package NQueens;

import java.util.*;
import backTrack.*;

public class NQueens implements Problem {

  private int maxN;
	private Map assignments = new HashMap();
	
  public NQueens (int n) throws IllegalArgumentException {
		if (n <= 0)
      throw new IllegalArgumentException
      	("NQueens(Problem): Illegal n ("+n+") <= 0");
    
    maxN = n;
	}
  
  
	public Problem.Aspect chooseAndExcludeAspect() throws Succeed {
		if (assignments.size() >= maxN) 
			throw new Succeed();
    return new Aspect (assignments.size());
	}
	
	
	public class Aspect implements Problem.Aspect {
		int rowWorkingOn;

		private Aspect (int rowWorkingOn) {
			this.rowWorkingOn = rowWorkingOn;
		}
		
		public boolean equals(Object o) {
			if (o == null)
				return false;
			if (this == o)
				return true;
			if (!(o instanceof Aspect))
				return false;
			
			return rowWorkingOn == ((Aspect)o).rowWorkingOn;
		}

		public int hashCode() {
			return rowWorkingOn;
		}
		
    public Iterator iterator () {
    	return new NQueenIterator ();
    }
			
   public String assignmentTo(Object o) {
   	  return "  On row "+rowWorkingOn+" place queen on column " + o;
    }
   
   public String toString () {
  	  return "NQueens.Aspect[rowWorkingOn"+rowWorkingOn+"]";
   }
  
   
   private class NQueenIterator implements Iterator {
    private int c = 0;
    	
    	private NQueenIterator() {
    		while (c < maxN && !isLegalInColumn(c))
    			c++;
    	}
    	
    	public boolean hasNext() {
    		return c < maxN;
    	}
    	
    	public Object next() {
    		int answer = c;
    		c++;
  		  while (c < maxN && !isLegalInColumn(c))
    			c++;
  		  return new Integer(answer);
    	}
    	
    	public void remove() {
    		throw new UnsupportedOperationException();
    	}
    	
		  private boolean isLegalInColumn(int c){
			  for (Iterator i = assignments.entrySet().iterator(); i.hasNext(); ){
			  	Map.Entry m = (Map.Entry)i.next();
				  int row    = ((Aspect) m.getKey())  .rowWorkingOn;
				  int column = ((Integer)m.getValue()).intValue();
				  if (c == column || Math.abs(c-column) == (rowWorkingOn-row))
					  return false;
			  }
			  return true;
		  }
		
    }
	}

  public Map getSolution () {
  	return new HashMap(assignments);
  }
  
  public void assignAspectValue(Problem.Aspect aspect, Object value) {
  	assignments.put(aspect,value);
  }
  
	public void removeAspectValue (Problem.Aspect aspect) {
  	assignments.remove(aspect);
  }
  
  public Problem copy () {
  	NQueens answer     = new NQueens (maxN);
  	answer.assignments = new HashMap(assignments);
  	return answer;
  }

  public String toString () {
  	return "NQueens[maxN="+maxN+";assignments="+assignments+"]";
  }
}

