package Satisfiability;

import java.util.*;
import backTrack.*;

public class Satisfiability implements Problem {

	static List ft = new ArrayList();
	static {
	  ft.add(new Boolean(false));
  	ft.add(new Boolean(true));
	}

	public interface Formula {
		  public Collection getVariables();
	  	public boolean    evaluate(Map assignment);
	  }
	
	private Formula    formula;
	private Collection variables;
	private Map        assignments = new HashMap();
	
  public Satisfiability (Formula formula, Collection variables) {
	  this.formula   = formula;
	  this.variables = new HashSet(variables);
	}
  
  
	public Problem.Aspect chooseAndExcludeAspect() throws Succeed,Fail {
		if (variables.size() == 0)
			if ( formula.evaluate(assignments) )
			  throw new Succeed();
			else
				throw new Fail();
		
		String v = (String)variables.iterator().next();
		variables.remove(v);
    return new Aspect(v);
	}
	
	
	public class Aspect implements Problem.Aspect {
		String variable;

		private Aspect (String variable) {
			this.variable = variable;
		}
		
		public String getVariable () {
			return variable;
		}
		public boolean equals(Object o) {
		  if (o == null)
				return false;
			if (this == o)
				return true;
			if (!(o instanceof Aspect))
				return false;
	
			return variable.equals(((Aspect)o).variable);
		}

		public int hashCode() {
			return variable.hashCode();
		}
		
    public Iterator iterator () {
     	return ft.iterator();
    }
			
    public String assignmentTo(Object o) {
    	return variable+"="+o;
    }
     
    public String toString () {
    	return "Satisfiability.Aspect[variable="+variable+"]";
    }
	}   
    
  public Map getSolution () {
  	return new HashMap(assignments);
  }
  
  public void assignAspectValue(Problem.Aspect aspect, Object value) {
  	assignments.put(aspect,value);
  }
  
	public void removeAspectValue(Problem.Aspect aspect) {
  	assignments.remove(aspect);
  }
  
  public Problem copy () {
  	Satisfiability answer = new Satisfiability (formula,variables);
  	answer.assignments    = new HashMap(assignments);
  	return answer;
  }

  public String toString () {
  	return "Satisfiability[formula="+formula+";assignments="+assignments+"]";
  }
}

