package Sudoku;

import java.util.*;
import backTrack.*;

public class Sudoku implements Problem {

	private Integer[][] puzzle = new Integer[9][9];

	public Sudoku() {
	}

	public Sudoku(int[][] puzzle) {
		for (int r = 0; r < 9; r++)
			for (int c = 0; c < 9; c++)
				this.puzzle[r][c] = new Integer(puzzle[r][c]);
	}

	public Problem.Aspect chooseAndExcludeAspect() throws Succeed {
		for (int r = 0; r < 9; r++)
			for (int c = 0; c < 9; c++)
				if (puzzle[r][c] == 0)
					return new Aspect(r, c);
		
		//No aspects left!
		throw new Succeed();
	}

	public class Aspect implements Problem.Aspect {
		private int row, col;

		private Aspect(int row, int col) {
			this.row = row;
			this.col = col;
		}

		public int getRow() {
			return row;
		}
		
		public int getCol() {
			return col;
		}
		
		public boolean equals(Object o) {
			if (o == null)
				return false;
			if (this == o)
				return true;
			if (!(o instanceof Aspect))
				return false;

			return row == ((Aspect) o).row && col == ((Aspect) o).col;
		}

		public int hashCode() {
			return 9 * row + col;
		}

		public Iterator iterator() {
			return new SudokuIterator();
		}

		public String assignmentTo(Object o) {
			return "  On [" + row + "," + col + "] assign " + o;
		}

		public String toString() {
			return "Sudoku.Aspect[row=" + row + ",col=" + col + "]";
		}

		private class SudokuIterator implements Iterator {
			private Integer[] aTry;

			private int on = 0;

			private SudokuIterator() {
				Set<Integer> tries = new HashSet<Integer>();
				for (int i = 1; i < 10; i++)
					tries.add(new Integer(i));
				for (int c = 0; c < 9; c++)
					tries.remove(puzzle[row][c]);
				for (int r = 0; r < 9; r++)
					tries.remove(puzzle[r][col]);
				for (int r = row / 3 * 3; r < row / 3 * 3 + 3; r++)
					for (int c = col / 3 * 3; c < col / 3 * 3 + 3; c++)
						tries.remove(puzzle[r][c]);
				aTry = new Integer[tries.size()];
				aTry = tries.toArray(aTry);
			}

			public boolean hasNext() {
				return on < aTry.length;
			}

			public Object next() {
				if (!hasNext())
					throw new NoSuchElementException();
				return aTry[on++];
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}

		}
	}

	public Map getSolution() {
		Map answer = new HashMap();
		for (int r = 0; r < 9; r++)
			for (int c = 0; c < 9; c++)
				answer.put(new Aspect(r,c), puzzle[r][c]);
		
		return answer;
	}

	public void assignAspectValue(Problem.Aspect aspect, Object value) {
		puzzle[((Aspect) aspect).row][((Aspect) aspect).col] = (Integer) value;
	}

	public void removeAspectValue(Problem.Aspect aspect) {
		puzzle[((Aspect) aspect).row][((Aspect) aspect).col] = 0;
	}

	public Problem copy() {
		Sudoku answer = new Sudoku();
		for (int r = 0; r < 9; r++)
			for (int c = 0; c < 9; c++)
				answer.puzzle[r][c] = puzzle[r][c];
		return answer;
	}

}
